###Install required R packages
#install.packages("nnet")
#install.packages("dfcrm")

###Load required R packages
library(nnet)
library(dfcrm)
library(pocrm)
library(binom)


###Load the function 'bpocrm' 
breast49<-function(pA,pB,skelA,tul,maxN,nstopA,nstopB,startA,cs,avar,popAdata){
 
      #if a single ordering is inputed as a vector, convert it to a matrix
	if(is.vector(skelA)) skelA=t(as.matrix(skelA));

	nord.tox = nrow(skelA);
	mprior.tox = rep(1/nord.tox, nord.tox);  # prior for each toxicity ordering

     post.tox<-function(a,p,y,n){
	s2=avar
	lik=1
	for(j in 1:length(p)){
		pj=p[j]**exp(a)
		lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
		}
	return(lik*exp(-0.5*a*a/s2));
    }

    #the posterior mean of ptox
    posttoxf <- function(a, p, y, n, j) { p[j]^(exp(a))*post.tox(a, p, y, n); }
	
    ###run pop A trial 	
    ncomb = ncol(skelA);   #number of combos
    yA=rep(0,ncomb);  #number of toxicity/responses at each dose level
    nA=rep(0,ncomb);  #number of treated patients at each dose level
    comb.curr = startA;  # current dose level	 
    ptox.hat = numeric(ncomb); # estimate of toxicity prob
    p.safety=numeric(ncomb)
    comb.selectA=rep(0,ncomb); # a vector of indicators for dose selection
    stopA=0; #indicate if trial stops early
    i=1	

while(i <= maxN)
    {
	# generate data for a new cohort of patients
		yA[comb.curr] = yA[comb.curr] + rbinom(1,1,pA[comb.curr]);
		nA[comb.curr] = nA[comb.curr] + 1;	

		safety=binom.confint(yA[1],nA[1],conf.level=cs,methods="agresti-coull")$lower	

		if(safety>tul){
			stopA=1
			break
			}	

		marginal.tox = rep(0, nord.tox);
		for(k in 1:nord.tox)
		{
			marginal.tox[k] = integrate(post.tox,lower=-Inf,upper=Inf, p=skelA[k,], y=yA,n=nA)$value;
		}
		
		postprob.tox = (marginal.tox*mprior.tox)/sum(marginal.tox*mprior.tox);

  		##Bayesian model selection, identify the model with the highest posterior prob
			if(nord.tox>1){ 
				mtox.sel = which.is.max(postprob.tox); 
			} else{
				mtox.sel = 1;
			}

		##calculate posterior mean of toxicity probability at each combo
			for(j in 1:ncomb){
				ptox.hat[j] = integrate(posttoxf,lower=-Inf,upper=Inf, skelA[mtox.sel,], yA, nA,j)$value/marginal.tox[mtox.sel]; 
			}
		
		distance=abs(ptox.hat-tul)
		comb.best=which.is.max(-distance)
		comb.curr=min(comb.best,comb.curr+1)

		if(nA[comb.curr]>=nstopA){
          		stopA<-0
          		break
        	}

		i=i+1
	    }
	 
	    if(stopA==0){
		comb.selectA[comb.curr]=comb.selectA[comb.curr]+1;
		}


if(popAdata=="no"){
	skelB<-getwm(q.skel,ptox.hat)
	if(is.vector(skelB)) skelB=t(as.matrix(skelB));
	mprior.tox=postprob.tox;
	ncomb = ncol(skelB);
	yB=ptox.hat; 
      nB=rep(1,ncomb);
	i=1
	}
if(popAdata=="yes"){
	skelB<-skelA
	if(is.vector(skelB)) skelB=t(as.matrix(skelB));
	mprior.tox=rep(1/nord.tox, nord.tox);
	ncomb = ncol(skelB);
	yB=yA; 
      nB=nA;
	i=sum(nA)+1
}

	nord.tox = nrow(skelB);
	popAprobs=postprob.tox
      ptox.hat = numeric(ncomb); # estimate of toxicity prob
      comb.selectB=rep(0,ncomb); # a vector of indicators for dose selection
      stopB=0; #indicate if trial stops earl


	###run pop B trial 	
while(i <= (maxN-sum(nA)))
    {
	# generate data for a new cohort of patients
		yB[comb.curr] = yB[comb.curr] + rbinom(1,1,pB[comb.curr]);
		nB[comb.curr] = nB[comb.curr] + 1;	

		safety=binom.confint(yB[1],nB[1],conf.level=cs,methods="agresti-coull")$lower	

		if(safety>tul){
			stopB=1
			break
			}	

		marginal.tox = rep(0, nord.tox);
		for(k in 1:nord.tox)
		{
			marginal.tox[k] = integrate(post.tox,lower=-Inf,upper=Inf, p=skelB[k,], y=yB,n=nB)$value;
		}
		
		postprob.tox = (marginal.tox*mprior.tox)/sum(marginal.tox*mprior.tox);

  		##Bayesian model selection, identify the model with the highest posterior prob
			if(nord.tox>1){ 
				mtox.sel = which.is.max(postprob.tox); 
			} else{
				mtox.sel = 1;
			}

		##calculate posterior mean of toxicity probability at each combo
			for(j in 1:ncomb){
				ptox.hat[j] = integrate(posttoxf,lower=-Inf,upper=Inf, skelB[mtox.sel,], yB, nB,j)$value/marginal.tox[mtox.sel]; 
			}

		
		
		distance=abs(ptox.hat-tul)
		comb.best=which.is.max(-distance)
		comb.curr=min(comb.best,comb.curr+1)

		if(nB[comb.curr]>=nstopB){
          		stopB<-0
          		break
        	}

		i=i+1
		}
	 
	    if(stopB==0){
		comb.selectB[comb.curr]=comb.selectB[comb.curr]+1;
		}

	return(list(comb.selectA=comb.selectA,tox.dataA=yA,pt.allocationA=nA,pordA=popAprobs,stopA=stopA, 
			comb.selectB=comb.selectB,tox.dataB=yB,pt.allocationB=nB,pordB=postprob.tox,stopB=stopB))
}
##########'bpocrm' end here




###Load the function 'bpocrm.sim' 
breast49sim<-function(pA,pB,skelA,tul,maxN,nstopA,nstopB,startA,cs,avar,popAdata,ntrial){
	ncomb=length(pA)
	
	MTD.selectA<-MTD.selectB<-toxA<-ptsA<-toxB<-ptsB<-matrix(0,nrow=ntrial,ncol=ncomb)
	nsA=nsB=0
	totalsize=totalsizeA=totalsizeB=rep(0,ntrial)
	
	for(j in 1:ntrial){
		result<-breast49(pA,pB,skelA,tul,maxN,nstopA,nstopB,startA,cs,avar,popAdata)
		MTD.selectA[j,]=result$comb.selectA
		toxA[j,]=result$tox.dataA
		ptsA[j,]=result$pt.allocationA
		MTD.selectB[j,]=result$comb.selectB
		toxB[j,]=result$tox.dataB
		ptsB[j,]=result$pt.allocationB
		totalsize[j]=sum(result$pt.allocationA,result$pt.allocationB)
		totalsizeA[j]=sum(result$pt.allocationA)
		totalsizeB[j]=sum(result$pt.allocationB)
		nsA=nsA+result$stopA
		nsB=nsB+result$stopB
	}
	#cat("Simulation results for partial order continual\n");
   	#cat("reassessment method targeting a DLT rate of", ttr,"\n\n");
	cat("True A DLT probability:\n");
	cat(round(pA,3), sep="\t",  "\n");
	cat("True B DLT probability:\n");
	cat(round(pB,3), sep="\t",  "\n");
	cat("A MTD selection percentage:\n");
	cat(formatC(colMeans(MTD.selectA)*100, digits=1, format="f"), sep="\t",  "\n");
	cat("B MTD selection percentage:\n");
    	cat(formatC(colMeans(MTD.selectB)*100, digits=2, format="f"), sep="\t",  "\n");
	cat("Average number of DLTs Pop A:\n");
	cat(formatC(colMeans(toxA), digits=1, format="f"), sep="\t",   "\n");
	cat("Average number of A pts treated:\n");
	cat(formatC(colMeans(ptsA), digits=1, format="f"), sep="\t",   "\n");
	cat("Average number of DLTs Pop B:\n");
	cat(formatC(colMeans(toxB), digits=1, format="f"), sep="\t",   "\n");
	cat("Average number of B pts treated:\n");
	cat(formatC(colMeans(ptsB), digits=1, format="f"), sep="\t",   "\n");
	cat("Distribution of A sample size:\n");
	cat(fivenum(totalsizeA), sep="\t",  "\n");
	cat("Distribution of B sample size:\n");
	cat(fivenum(totalsizeB), sep="\t",  "\n");
	cat("Distribution of total sample size:\n");
	cat(fivenum(totalsize), sep="\t",  "\n");
	cat("Percentage of trials stopped for safety in A:\n");
	cat(nsA/ntrial*100, "\n");
	cat("Percentage of trials stopped for safety in B:\n");
	cat(nsB/ntrial*100, "\n");
	}
##########'bpocrm.sim' end here



#####Specify the total number of combinations
d<-4

#####Specify the number of possible toxicity orderings
s<-2  

q.skel<-matrix(nrow=s,ncol=d)
q.skel[1,]<-c(1:4)
q.skel[2,]<-c(1,3,2,4)

###Specify a set of toxicity skeleton values
skel<-c(0.25,0.35,0.46,0.56)#getprior(0.0625,0.25,1,d)

skelA<-getwm(q.skel,skel)

###True toxicity probability scenarios
pA1<-c(0.02,0.09,0.09,0.15)
pA2<-c(0.10,0.25,0.15,0.35)
pA3<-c(0.10,0.15,0.25,0.35)
pA4<-c(0.20,0.37,0.50,0.60)
pA5<-c(0.50,0.60,0.60,0.70)


pA6<-c(0.14,0.35,0.22,0.50)
pB6<-c(0.04,0.25,0.12,0.40)


#######################################
#
#	Input
#
#	p0 = true toxicity probabilities 
#	p.skel = matrix of toxicity skeleton values
#	tul = target toxicity rate
# 	cohortsize = size of each cohort inclusion
# 	nchort = number of cohorts in a single trial
#	start.comb = starting combination
#	ntrial = number of simulated trials
#	cs = stopping threshold for safety
#	 
########################################

tul=0.25       ##target toxicity rate 
maxN=55        ##number of cohorts
startA=1       ##starting combination
nstopA=6       ##Number of patients needed on one combination to stop the trial
nstopB=30
ntrial=5000    ##number of simulated trials 
cs=0.9         ##stopping threshold for safety
avar=1.3
popAdata="yes"

##simulate a single trial
#result=breast49(pA,pB,skelA,tul,maxN,nstopA,nstopB,startA,cs,avar,popAdata)
#result

pA<-pA3
pB<-pA1
set.seed(580)  ##random seed
system.time(breast49sim(pA,pB,skelA,tul,maxN,nstopA,nstopB,startA,cs,avar,popAdata,ntrial))



